==> snippet_library-1.2.6.tgz: package/app/components/questions/dropdown/dropdown.component.js <==
import dropdownTemplate from './dropdown.template.html'

/* @ngInject */
let dropdownComponent = {
  template: dropdownTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: ($scope, $rootScope, $http, $state, $stateParams, $localStorage, $timeout, $log, xxtea, exerciseService, appConstants, popupService) => {

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    $scope.step = 0 // Current question (step)
    $scope.attempt = 0 // Attempt number
    $scope.allAnswers = [] // All user answers
    $scope.attempts = [] // All user attempts
    $scope.correct = false // Is user answer correct
    $scope.queans = {} // User answers bindings
    $scope.tryAgain = false // Is in try again state
    $scope.finalAttempt = false // Is in final attempt state
    $scope.time_finished = '' // Time when user started the exercise
    $scope.time_attempted = new Date().getTime() // Time when the user finished the exercise

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })

    /**
         * Switch to next question or finish exercise if the step is last
         * @returns {void}
         */
    $scope.nextStep = () => {
      if ($scope.step + 1 === $scope.exercise.interaction.questions.length) {
        $scope.time_finished = new Date().getTime()
        $scope.finishExercise()
      } else {
        popupService.hide()
        $scope.step++
        $scope.correct = false
        $scope.attempt = 0
        $scope.tryAgain = false
        $scope.finalAttempt = false
        $scope.attempts = []
        $scope.validation = []
      }

      $scope.currentActiveQuestion = $scope.step + 1
      $localStorage.currentActiveQuestion = $scope.step + 1
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.question = $scope.exercise.interaction.questions[$scope.step]
      $scope.answers = $scope.question.answers // Get answers
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
    }

    /**
         * Checks all answers and show the feedback in popup directive
         * @returns {void}
         */
    $scope.check = () => {
      if ($scope.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      } else {
        $scope.userAnswers = $scope.queans[$scope.step]
        $scope.validation = []
        $scope.correctAnswers = []
        $scope.attempt++

        let cQuestion = $scope.exercise.interaction.correct.find((q) => { return $scope.question.question_id === q.question_id })
        if (cQuestion) {
          let correctAnswers = cQuestion.answers

          angular.forEach(correctAnswers, (v, k) => {
            $scope.v = v
            let userAnswer = $scope.userAnswers ? $scope.userAnswers[k + 1] : undefined
            let isCorrect = false
            if (userAnswer) {
              let encryptedUserAnswer = xxtea.encryptToBase64(userAnswer.replace(/_/g, ' '), appConstants.tea)
              isCorrect = v.includes(encryptedUserAnswer)
              if (isCorrect) {
                $scope.correctAnswers.push(k)
              }
            }
            $scope.validation[k + 1] = isCorrect
          })

          let isCorrect = $scope.correctAnswers.length === $scope.question.answers.length
          if (isCorrect) {
            popupService.show('Very good!', 'happy-smile', true)
            $scope.question.answered = true
            $scope.correct = true
          } else {
            if ($scope.attempt === 1) {
              popupService.show('Some of your answers are incorrect. These have been left in place.', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 2) {
              popupService.show('Some of your answers are incorrect. These have been left in place.<br>Final chance!', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 3) {
              let htmlFeedback = correctAnswers.map((v, k) => { return (k + 1) + '. ' + xxtea.decryptFromBase64(v[0], appConstants.tea) }).join('<br/>')
              popupService.show('Sorry, your answer is not correct. The correct answers were: <br/>' + htmlFeedback, 'sad-smile', false)
              $scope.correct = true
              $scope.finalAttempt = true
              $scope.question.answered = true
            }

          }
          $scope.saveAnswer($scope.question.question_id, $scope.userAnswers, isCorrect, $scope.attempt)
        }
      }
    }

    /**
         * Pushes the attempts to array (Saves answer)
         * @param {int} questionId Question ID to push the attempt
         * @param {array<object>} answers The user answers array
         * @param {boolean} correct If the answer/answers is correct
         * @param {int} attemptNumber The user attempt number
         * @returns {void}
         */
    $scope.saveAnswer = (questionId, answers, correct, attemptNumber) => {
      const attempt = {
        'answer': answers,
        'correct': correct,
        'attempt_number': attemptNumber
      }

      let matchQuestion = $scope.allAnswers.find((q) => { return q.question === questionId })
      if ($scope.allAnswers.length > 0 && matchQuestion) {
        matchQuestion.attempts.push(attempt)
      } else {
        $scope.attempts.push(attempt)
        $scope.allAnswers.push({ 'question': questionId, 'attempts': $scope.attempts })
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      let allAnswersCount = 0
      angular.forEach($scope.exercise.interaction.questions, function (q) { allAnswersCount += q.answers.length })
      let correctAnswersCount = $scope.correctAnswers.length // Correct answers counter

      // Get correct answers count
      //angular.forEach($scope.allAnswers, (answer) => { correctAnswersCount += answer.attempts.some(function (item) { return item.correct; }) ? 1 : 0; });

      const score = Math.round(((correctAnswersCount / allAnswersCount) * 100))
      const data = {
        'score': score,
        'user': 24275,
        'answers': $scope.allAnswers,
        'time_attempted': $scope.time_attempted,
        'time_finished': $scope.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      // $http.post(appConstants.api + '/exerciseStoreScore', data).then(() => {
      //   let nextExercise = undefined
      //   if ($localStorage.playlist) {
      //     if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
      //       nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
      //     }
      //   }
      //
      //   exerciseService.setVal({
      //     'score': score,
      //     'totalQuestions': allAnswersCount,
      //     'correctAnswer': correctAnswersCount,
      //     'nextExercise': nextExercise
      //   })
      //
      //   $state.go('result') // Go to result page
      // }, $scope.errorCallback)

      let nextExercise = undefined
      exerciseService.setVal({
        'score': score,
        'totalQuestions': allAnswersCount,
        'correctAnswer': correctAnswersCount,
        'nextExercise': nextExercise
      })
      $state.go('result') // Go to result page
    }

    /**
         * Retry the exercise
         * @returns {void}
         */
    $scope.try = function () {
      popupService.hide()
      $scope.validation = []
      $scope.tryAgain = false
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      if (!$scope.correct && !$scope.tryAgain) {
        return type === 'text' ? 'Check' : $scope.check()
      } else if ($scope.attempt > 0 && $scope.tryAgain) {
        return type === 'text' ? 'Try again' : $scope.try()
      } else if ($scope.correct || $scope.attempt === 3) {
        return type === 'text' ? 'Next' : $scope.nextStep()
      }
    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      if ($stateParams.id) {
        $http({ method: 'GET', url: appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid })
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'dropdown', id: $stateParams.id, sid: $stateParams.sid }

          }, errorCallback)
      } else { // If we have loaded playlist then load the exercise from playlist
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default dropdownComponent

==> snippet_library-2.0.7.tgz: package/app/components/questions/dropdown/dropdown.component.js <==
import dropdownTemplate from './dropdown.template.html'

/* @ngInject */
let dropdownComponent = {
  template: dropdownTemplate,
  bindings: {
    questionsId: '=',
    sid: '='
  },
  controller: ($scope, $compile, $element, $location, $rootScope, $http, $state, $stateParams, $localStorage, $timeout, $log, xxtea, exerciseService, appConstants, popupService) => {

    /**
         * Error callback for HTTP response
         * @param {object} response The HTTP response object
         * @returns {void}
         */
    function errorCallback(response) {
      if (response.status === 401) {
        $log.error({ status: 'false', data: 'Bad token.' })
      } else if (response.status === 404) {
        $log.error({ status: 'false', data: 'Bad request.' })
      } else if (response.status === 405) {
        $log.error({ status: 'false', data: 'Method not allowed.' })
      } else {
        $log.error({ status: 'false', data: response.statusText })
      }

    };

    $scope.step = 0 // Current question (step)
    $scope.attempt = 0 // Attempt number
    $scope.allAnswers = [] // All user answers
    $scope.attempts = [] // All user attempts
    $scope.correct = false // Is user answer correct
    $scope.queans = {} // User answers bindings
    $scope.tryAgain = false // Is in try again state
    $scope.finalAttempt = false // Is in final attempt state
    $scope.time_finished = '' // Time when user started the exercise
    $scope.time_attempted = new Date().getTime() // Time when the user finished the exercise

    // Popup
    $scope.popupVisible = false
    let popupVisibilityChange = $rootScope.$on('popup.changed', function (elem, data) {
      $scope.popupVisible = data.active
    })

    /**
         * Switch to next question or finish exercise if the step is last
         * @returns {void}
         */
    $scope.nextStep = () => {
      if ($scope.step + 1 === $scope.exercise.interaction.questions.length) {
        $scope.time_finished = new Date().getTime()
        $scope.finishExercise()
      } else {
        popupService.hide()
        $scope.step++
        $scope.correct = false
        $scope.attempt = 0
        $scope.tryAgain = false
        $scope.finalAttempt = false
        $scope.attempts = []
        $scope.validation = []
      }

      $scope.currentActiveQuestion = $scope.step + 1
      $localStorage.currentActiveQuestion = $scope.step + 1
    }

    /**
         * Setting up the exercise
         * @param {object} exerciseData exercise object
         * @returns {void}
         */
    function setupExercise(exerciseData) {
      $scope.exercise = exerciseData
      $scope.question = $scope.exercise.interaction.questions[$scope.step]
      $scope.answers = $scope.question.answers // Get answers
      $localStorage.questionCount = $scope.exercise.interaction.questions.length // Get questions count
    }

    /**
         * Checks all answers and show the feedback in popup directive
         * @returns {void}
         */
    $scope.check = () => {
      if ($scope.question.answered === true) {
        popupService.show('You have already answered this question!', 'semi-smile', false)
      } else {
        $scope.userAnswers = $scope.queans[$scope.step]
        $scope.validation = []
        $scope.correctAnswers = []
        $scope.attempt++

        let cQuestion = $scope.exercise.interaction.correct.find((q) => { return $scope.question.question_id === q.question_id })
        if (cQuestion) {
          let correctAnswers = cQuestion.answers

          angular.forEach(correctAnswers, (v, k) => {
            $scope.v = v
            let userAnswer = $scope.userAnswers ? $scope.userAnswers[k + 1] : undefined
            let isCorrect = false
            if (userAnswer) {
              let encryptedUserAnswer = xxtea.encryptToBase64(userAnswer.replace(/_/g, ' '), appConstants.tea)
              isCorrect = v.includes(encryptedUserAnswer)
              if (isCorrect) {
                $scope.correctAnswers.push(k)
              }
            }
            $scope.validation[k + 1] = isCorrect
          })

          let isCorrect = $scope.correctAnswers.length === $scope.question.answers.length
          if (isCorrect) {
            popupService.show('Very good!', 'happy-smile', true)
            $scope.question.answered = true
            $scope.correct = true
          } else {
            if ($scope.attempt === 1) {
              popupService.show('Some of your answers are incorrect. These have been left in place.', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 2) {
              popupService.show('Some of your answers are incorrect. These have been left in place.<br>Final chance!', 'semi-smile', false)
              $scope.tryAgain = true
            } else if ($scope.attempt === 3) {
              let htmlFeedback = correctAnswers.map((v, k) => { return (k + 1) + '. ' + xxtea.decryptFromBase64(v[0], appConstants.tea) }).join('<br/>')
              popupService.show('Sorry, your answer is not correct. The correct answers were: <br/>' + htmlFeedback, 'sad-smile', false)
              $scope.correct = true
              $scope.finalAttempt = true
              $scope.question.answered = true
            }

          }
          $scope.saveAnswer($scope.question.question_id, $scope.userAnswers, isCorrect, $scope.attempt)
        }
      }
    }

    /**
         * Pushes the attempts to array (Saves answer)
         * @param {int} questionId Question ID to push the attempt
         * @param {array<object>} answers The user answers array
         * @param {boolean} correct If the answer/answers is correct
         * @param {int} attemptNumber The user attempt number
         * @returns {void}
         */
    $scope.saveAnswer = (questionId, answers, correct, attemptNumber) => {
      const attempt = {
        'answer': answers,
        'correct': correct,
        'attempt_number': attemptNumber
      }

      let matchQuestion = $scope.allAnswers.find((q) => { return q.question === questionId })
      if ($scope.allAnswers.length > 0 && matchQuestion) {
        matchQuestion.attempts.push(attempt)
      } else {
        $scope.attempts.push(attempt)
        $scope.allAnswers.push({ 'question': questionId, 'attempts': $scope.attempts })
      }
    }

    /**
         * Sends an POST request to score API and finishes the exercise
         * @returns {void}
         */
    $scope.finishExercise = () => {
      let allAnswersCount = 0
      angular.forEach($scope.exercise.interaction.questions, function (q) { allAnswersCount += q.answers.length })
      let correctAnswersCount = $scope.correctAnswers.length // Correct answers counter

      // Get correct answers count
      //angular.forEach($scope.allAnswers, (answer) => { correctAnswersCount += answer.attempts.some(function (item) { return item.correct; }) ? 1 : 0; });

      const score = Math.round(((correctAnswersCount / allAnswersCount) * 100))
      const data = {
        'score': score,
        'answers': $scope.allAnswers,
        'time_attempted': $scope.time_attempted,
        'time_finished': $scope.time_finished,
        'exercise_id': $scope.exercise.exercise_id,
        'playlist_id': $scope.exercise.playlist.playlist_id,
        'context': $scope.exercise.context.context,
        'context_id': $scope.exercise.context.context_id,
        'id': $scope.exercise.exercise_id,
        'group_id': $scope.exercise.group_id
      }

      $http.post(appConstants.api + '/exercise/score/store', data, appConstants.http_config).then(() => {
      }, $scope.errorCallback)

      let nextExercise = undefined
      if ($localStorage.playlist) {
        if ($localStorage.currentActiveExercise !== $localStorage.exerciseCount) {
          nextExercise = $localStorage.playlist[$localStorage.currentActiveExercise].exercise_id
        }
      }
      exerciseService.setVal({
        'score': score,
        'totalQuestions': allAnswersCount,
        'correctAnswer': correctAnswersCount,
        'nextExercise': nextExercise
      })
      if ($scope.loc !== 'practice') {
        $state.go('result') // Go to result page
      } else {
        $element.replaceWith($compile('<result></result>')($scope))
      }
    }

    /**
         * Retry the exercise
         * @returns {void}
         */
    $scope.try = function () {
      popupService.hide()
      $scope.validation = []
      $scope.tryAgain = false
    }

    /**
         * Checks current action that have to be displayed
         * @param {string} type text or function
         * @returns {object} string or function
         */
    $scope.currentAction = (type) => {
      if (!$scope.correct && !$scope.tryAgain) {
        return type === 'text' ? 'Check' : $scope.check()
      } else if ($scope.attempt > 0 && $scope.tryAgain) {
        return type === 'text' ? 'Try again' : $scope.try()
      } else if ($scope.correct || $scope.attempt === 3) {
        return type === 'text' ? 'Next' : $scope.nextStep()
      }
    }

    // Get exercise data and setup
    $scope.$watch('[$ctrl.questionsId,$ctrl.sid]', (n, o) => {
      $scope.loc = $location.absUrl().replace('/' + $scope.location, '').split('/').slice(-1).pop()
      if ($stateParams.id) {
        $http.get(appConstants.api + '/exercise/' + $scope.$ctrl.questionsId + '/' + $scope.$ctrl.sid, appConstants.http_config)
          .then((response) => {
            setupExercise(response.data)
            $localStorage.$reset()
            $localStorage.exercises = []
            $localStorage.questionCount = $scope.exercise.interaction.questions.length
            $localStorage.questions = $scope.exercise.interaction.questions
            $localStorage.currentActiveExercise = 1
            $localStorage.currentActiveQuestion = $scope.step + 1

            // Assign new data
            $localStorage.playlist = [response.data]
            $localStorage.exerciseCount = $localStorage.playlist.length
            $localStorage.currentExercise = response.data.exercise_id
            $localStorage.currentActiveExercise = 1
            $localStorage.questionLink = { type: 'dropdown', id: $stateParams.id, sid: $stateParams.sid }
            if ($scope.loc !== 'practice') {
              $timeout(() => { $state.go('exercise') })
            }
          }, errorCallback)
      } else {
        setupExercise($localStorage.playlist.find((item) => item.exercise_id === $localStorage.currentExercise))
      }
    })
  }

}
export default dropdownComponent
